#!/bin/bash
########################################################################
# build_cmake.sh
########################################################################

####################################
# Useful variables
prog=${0##*/}
thisdir=$(cd "$(dirname "${0}")" && pwd -P)

####################################
# Useful functions.

# Report usage.
usage() {
   echo "USAGE: ${prog} <product_dir> [tar]"
}

# Make sure we have UPS and ssibuildshims set up.
get_ssibuildshims() {
  [[ -n "${UPS_DIR}" ]] || { echo "ERROR: UPS not set up" 1>&2; exit 1; }
  source $("${UPS_DIR}/bin/ups" setup ${SETUP_UPS})
  setup ssibuildshims ${ssibuildshims_version} -z ${product_dir} || { \
    echo "ERROR: failed to set up ssibuildshims ${ssibuildshims_version}" 1>&2; \
    exit 1; }
}

# Make the distribution and leave if successful.
maybe_make_distribution() {
  if [ "${maketar}" = "tar" ] && [ -d "${pkgdir}/bin" ]; then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball \
      "${product_dir}" ${package} ${pkgver} NO_QUAL
	  exit 0
  fi
}

# Attempt to set up a more recent GCC if we need it.
function maybe_setup_gcc() {
  [[ "$flvr" == Linux* ]] || return
  if ver_greater 4.8.0 $(gcc -dumpversion); then
    # Need special GCC that understands --static-stdlibc++ and
    # --static-libgcc.
    gccprod=$(latest_version -t ${product_dir} --ge 6.1.0 gcc -z "${product_dir}" -z "${PRODUCTS}")
    (( $? == 0 )) || \
      ssi_die "On systems with GCC < 4.8.0, need access to GCC >= 6.1. Best version found was ${best_version}"
    eval setup -B ${gccprod} || ssi_die "Unable to set up ${gccprod}"
    export LDFLAGS="${LDFLAGS:+${LDFLAGS} }-static-libstdc++ -static-libgcc"
  fi
}

########################################################################
# Main.
########################################################################

####################################
# Arguments.
product_dir=${1}
maketar=${2}

# Basic check.
[[ -n "${product_dir}" ]] || \
  { printf "ERROR: please specify the local product directory\n$(usage)\n" 1>&2; \
  exit 1; }

####################################
# Package name and version information.
package=cmake
origpkgver=v3_17_1
pkgver=${origpkgver}
ssibuildshims_version=v2_00_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=${package}-${pkgdotver}.tar.gz

# Obtain access to useful functions.
get_ssibuildshims
source ssi_functions

# Define globals.
source define_basics NO_QUAL NO_DEBUG_SOURCE

# Possible short-circuit.
maybe_make_distribution

echo "Building ${package} for ${OS}-${plat} (flavor ${flvr})"

####################################
# Ensure directories.
mkdir -p "${pkgdir}" || \
  ssi_die "failed to create installation directory ${pkgdir}"
mkdir -p "${blddir}" || ssi_die "failed to create build directory ${blddir}"
# Remove any existing sourcedir to avoid patch conflicts.
rm -rf "${sourcedir}" && mkdir -p ${sourcedir} || \
  ssi_die "failed to create source directory ${sourcedir}"
####################################

# Attempt to set up a more recent GCC if we need it.
maybe_setup_gcc

# Print active products.
echo
ups active
echo

cd "${sourcedir}" || ssi_die "unable to cd to source directory ${sourcedir}"

# Expand and patch the source.
prepare_source -o ${package}-${pkgdotver}-fixes.patch \
  "${tardir}/${pkgtarfile}" \
  fix-find-frameworks.patch || exit $?

[ -n "${DEBUG_PATCHES}" ] && exit 1

# Ascertain and configure parallelism.
ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

cd "${blddir}" || ssi_die "unable to cd to build directory ${blddir}"

####################################
# Configure, build, and install.
set -x

# Configure and execute bootstrap.
${sourcedir}/${package}-${pkgdotver}/bootstrap \
  --prefix=${pkgdir} \
  --parallel=${ncore}

# Build.
make -j ${ncore} || ssi_die "make failed"

# Test.
# ctest -j ${ncore} || ssi_die "tests failed"

# Install.
make -j ${ncore} install  || ssi_die "'make install' failed"

set +x

# Verify install.
[[ -d "${pkgdir}/bin" ]] || \
  ssi_die "failed to create ${pkgdir}/bin:\n" \
  "      something is very wrong"
####################################

# Declare the installed package.
ups exist ${package} ${pkgver} -z ${product_dir} || \
  ${SSIBUILDSHIMS_DIR}/bin/declare_product "${product_dir}" \
  ${package} ${pkgver} NO_QUAL || \
  ssi_die "Unable to declare product to UPS."

echo "Finished building ${package} ${pkgver}"

# Attempt to set up the package.
setup ${package} ${pkgver} -z "${product_dir}:${PRODUCTS}" || \
  ssi_die "unable to set up newly-built product."

# Show that we found it.
which cmake

echo "${package} is installed at ${pkgdir}"

# This must be last.
maybe_make_distribution

exit 0
